#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "cabinet.h"

#define die(...)		{ fprintf(stderr, __VA_ARGS__); exit(1); }

void pcab_info(struct cab_info *info)
{
	printf("paddr: %p\n",		(void *) info->paddr);
	printf("pf_paddr: %p\n",	(void *) info->pf_paddr);
	printf("pte_paddr: %p\n",	(void *) info->pte_paddr);
	printf("pmd_paddr: %p\n",	(void *) info->pmd_paddr);
	printf("pud_paddr: %p\n",	(void *) info->pud_paddr);
	printf("pgd_paddr: %p\n",	(void *) info->pgd_paddr);
	printf("modified: %s\n",	info->modified ? "yes" : "no");
	printf("refcount: %d\n",	info->refcount);
}

int main(int argc, char **argv)
{
	pid_t pid = -1;
	unsigned long vaddr = (unsigned long) &pid;
	struct cab_info inventory;

	switch (argc) {
	case 3:
		pid = atoi(argv[2]);
		if (!pid && strcmp(argv[2], "0"))
			die("error parsing pid: %s\n", argv[2]);
	case 2:
		vaddr = strtoul(argv[1], NULL, 16);
	case 1:
		break;
	default:
		die("usage: %s [<vaddr> [<pid>]] (as root)\n", argv[0]);
	}

	printf("inspecting cabinet for %p of pid=%d\n", (void *) vaddr, pid);

	if (inspect_cabinet(pid, vaddr, &inventory))		
		die("error occured with syscall: %s\n", strerror(errno));
	
	pcab_info(&inventory);

	return 0;
}
